@@ -2,6 +2,8 @@
 #define SASS_EMITTER_H
 
 #include <string>
+#include "sass.hpp"
+#include "sass/base.h"
 #include "source_map.hpp"
 #include "ast_fwd_decl.hpp"
 
@@ -11,7 +13,7 @@ namespace Sass {
   class Emitter {
 
     public:
-      Emitter(Context* ctx);
+      Emitter(struct Sass_Output_Options& opt);
       virtual ~Emitter() { }
 
     protected:
@@ -23,17 +25,19 @@ namespace Sass {
       // proxy methods for source maps
       void add_source_index(size_t idx);
       void set_filename(const std::string& str);
-      void add_open_mapping(AST_Node* node);
-      void add_close_mapping(AST_Node* node);
-      std::string generate_source_map(Context &ctx);
+      void add_open_mapping(const AST_Node_Ptr node);
+      void add_close_mapping(const AST_Node_Ptr node);
+      void schedule_mapping(const AST_Node_Ptr node);
+      std::string render_srcmap(Context &ctx);
       ParserState remap(const ParserState& pstate);
 
     public:
-      Context* ctx;
+      struct Sass_Output_Options& opt;
       size_t indentation;
       size_t scheduled_space;
       size_t scheduled_linefeed;
       bool scheduled_delimiter;
+      AST_Node_Ptr scheduled_mapping;
 
     public:
       // output strings different in comments
@@ -52,9 +56,9 @@ namespace Sass {
       // return buffer as std::string
       std::string get_buffer(void);
       // flush scheduled space/linefeed
-      Output_Style output_style(void);
+      Sass_Output_Style output_style(void) const;
       // add outstanding linefeed
-      void finalize(void);
+      void finalize(bool final = true);
       // flush scheduled space/linefeed
       void flush_schedules(void);
       // prepend some text or token to the buffer
@@ -66,7 +70,7 @@ namespace Sass {
       void append_wspace(const std::string& text);
       // append some text or token to the buffer
       // this adds source-mappings for node start and end
-      void append_token(const std::string& text, AST_Node* node);
+      void append_token(const std::string& text, const AST_Node_Ptr node);
 
     public: // syntax sugar
       void append_indentation();
@@ -75,8 +79,8 @@ namespace Sass {
       void append_special_linefeed(void);
       void append_optional_linefeed(void);
       void append_mandatory_linefeed(void);
-      void append_scope_opener(AST_Node* node = 0);
-      void append_scope_closer(AST_Node* node = 0);
+      void append_scope_opener(AST_Node_Ptr node = 0);
+      void append_scope_closer(AST_Node_Ptr node = 0);
       void append_comma_separator(void);
       void append_colon_separator(void);
       void append_delimiter(void);
