@@ -21,7 +21,7 @@ namespace Sass {
   Inspect::~Inspect() { }
 
   // statements
-  void Inspect::operator()(Block* block)
+  void Inspect::operator()(Block_Ptr block)
   {
     if (!block->is_root()) {
       add_open_mapping(block);
@@ -39,19 +39,23 @@ namespace Sass {
 
   }
 
-  void Inspect::operator()(Ruleset* ruleset)
+  void Inspect::operator()(Ruleset_Ptr ruleset)
   {
-    ruleset->selector()->perform(this);
-    ruleset->block()->perform(this);
+    if (ruleset->selector()) {
+      ruleset->selector()->perform(this);
+    }
+    if (ruleset->block()) {
+      ruleset->block()->perform(this);
+    }
   }
 
-  void Inspect::operator()(Keyframe_Rule* rule)
+  void Inspect::operator()(Keyframe_Rule_Ptr rule)
   {
-    if (rule->selector()) rule->selector()->perform(this);
+    if (rule->name()) rule->name()->perform(this);
     if (rule->block()) rule->block()->perform(this);
   }
 
-  void Inspect::operator()(Bubble* bubble)
+  void Inspect::operator()(Bubble_Ptr bubble)
   {
     append_indentation();
     append_token("::BUBBLE", bubble);
@@ -60,7 +64,7 @@ namespace Sass {
     append_scope_closer();
   }
 
-  void Inspect::operator()(Media_Block* media_block)
+  void Inspect::operator()(Media_Block_Ptr media_block)
   {
     append_indentation();
     append_token("@media", media_block);
@@ -71,7 +75,7 @@ namespace Sass {
     media_block->block()->perform(this);
   }
 
-  void Inspect::operator()(Supports_Block* feature_block)
+  void Inspect::operator()(Supports_Block_Ptr feature_block)
   {
     append_indentation();
     append_token("@supports", feature_block);
@@ -80,7 +84,7 @@ namespace Sass {
     feature_block->block()->perform(this);
   }
 
-  void Inspect::operator()(At_Root_Block* at_root_block)
+  void Inspect::operator()(At_Root_Block_Ptr at_root_block)
   {
     append_indentation();
     append_token("@at-root ", at_root_block);
@@ -89,7 +93,7 @@ namespace Sass {
     at_root_block->block()->perform(this);
   }
 
-  void Inspect::operator()(Directive* at_rule)
+  void Inspect::operator()(Directive_Ptr at_rule)
   {
     append_indentation();
     append_token(at_rule->keyword(), at_rule);
@@ -112,7 +116,7 @@ namespace Sass {
     }
   }
 
-  void Inspect::operator()(Declaration* dec)
+  void Inspect::operator()(Declaration_Ptr dec)
   {
     if (dec->value()->concrete_type() == Expression::NULL_VAL) return;
     bool was_decl = in_declaration;
@@ -120,13 +124,14 @@ namespace Sass {
     if (output_style() == NESTED)
       indentation += dec->tabs();
     append_indentation();
-    dec->property()->perform(this);
+    if (dec->property())
+      dec->property()->perform(this);
     append_colon_separator();
 
     if (dec->value()->concrete_type() == Expression::SELECTOR) {
-      Memory_Manager mem;
-      Listize listize(mem);
-      dec->value()->perform(&listize)->perform(this);
+      Listize listize;
+      Expression_Obj ls = dec->value()->perform(&listize);
+      ls->perform(this);
     } else {
       dec->value()->perform(this);
     }
@@ -141,7 +146,7 @@ namespace Sass {
     in_declaration = was_decl;
   }
 
-  void Inspect::operator()(Assignment* assn)
+  void Inspect::operator()(Assignment_Ptr assn)
   {
     append_token(assn->variable(), assn);
     append_colon_separator();
@@ -153,7 +158,7 @@ namespace Sass {
     append_delimiter();
   }
 
-  void Inspect::operator()(Import* import)
+  void Inspect::operator()(Import_Ptr import)
   {
     if (!import->urls().empty()) {
       append_token("@import", import);
@@ -161,9 +166,9 @@ namespace Sass {
 
       import->urls().front()->perform(this);
       if (import->urls().size() == 1) {
-        if (import->media_queries()) {
+        if (&import->import_queries()) {
           append_mandatory_space();
-          import->media_queries()->perform(this);
+          import->import_queries()->perform(this);
         }
       }
       append_delimiter();
@@ -174,9 +179,9 @@ namespace Sass {
 
         import->urls()[i]->perform(this);
         if (import->urls().size() - 1 == i) {
-          if (import->media_queries()) {
+          if (&import->import_queries()) {
             append_mandatory_space();
-            import->media_queries()->perform(this);
+            import->import_queries()->perform(this);
           }
         }
         append_delimiter();
@@ -184,7 +189,7 @@ namespace Sass {
     }
   }
 
-  void Inspect::operator()(Import_Stub* import)
+  void Inspect::operator()(Import_Stub_Ptr import)
   {
     append_indentation();
     append_token("@import", import);
@@ -193,7 +198,7 @@ namespace Sass {
     append_delimiter();
   }
 
-  void Inspect::operator()(Warning* warning)
+  void Inspect::operator()(Warning_Ptr warning)
   {
     append_indentation();
     append_token("@warn", warning);
@@ -202,7 +207,7 @@ namespace Sass {
     append_delimiter();
   }
 
-  void Inspect::operator()(Error* error)
+  void Inspect::operator()(Error_Ptr error)
   {
     append_indentation();
     append_token("@error", error);
@@ -211,7 +216,7 @@ namespace Sass {
     append_delimiter();
   }
 
-  void Inspect::operator()(Debug* debug)
+  void Inspect::operator()(Debug_Ptr debug)
   {
     append_indentation();
     append_token("@debug", debug);
@@ -220,14 +225,14 @@ namespace Sass {
     append_delimiter();
   }
 
-  void Inspect::operator()(Comment* comment)
+  void Inspect::operator()(Comment_Ptr comment)
   {
     in_comment = true;
     comment->text()->perform(this);
     in_comment = false;
   }
 
-  void Inspect::operator()(If* cond)
+  void Inspect::operator()(If_Ptr cond)
   {
     append_indentation();
     append_token("@if", cond);
@@ -242,7 +247,7 @@ namespace Sass {
     }
   }
 
-  void Inspect::operator()(For* loop)
+  void Inspect::operator()(For_Ptr loop)
   {
     append_indentation();
     append_token("@for", loop);
@@ -255,7 +260,7 @@ namespace Sass {
     loop->block()->perform(this);
   }
 
-  void Inspect::operator()(Each* loop)
+  void Inspect::operator()(Each_Ptr loop)
   {
     append_indentation();
     append_token("@each", loop);
@@ -270,7 +275,7 @@ namespace Sass {
     loop->block()->perform(this);
   }
 
-  void Inspect::operator()(While* loop)
+  void Inspect::operator()(While_Ptr loop)
   {
     append_indentation();
     append_token("@while", loop);
@@ -279,7 +284,7 @@ namespace Sass {
     loop->block()->perform(this);
   }
 
-  void Inspect::operator()(Return* ret)
+  void Inspect::operator()(Return_Ptr ret)
   {
     append_indentation();
     append_token("@return", ret);
@@ -288,7 +293,7 @@ namespace Sass {
     append_delimiter();
   }
 
-  void Inspect::operator()(Extension* extend)
+  void Inspect::operator()(Extension_Ptr extend)
   {
     append_indentation();
     append_token("@extend", extend);
@@ -297,7 +302,7 @@ namespace Sass {
     append_delimiter();
   }
 
-  void Inspect::operator()(Definition* def)
+  void Inspect::operator()(Definition_Ptr def)
   {
     append_indentation();
     if (def->type() == Definition::MIXIN) {
@@ -312,7 +317,7 @@ namespace Sass {
     def->block()->perform(this);
   }
 
-  void Inspect::operator()(Mixin_Call* call)
+  void Inspect::operator()(Mixin_Call_Ptr call)
   {
     append_indentation();
     append_token("@include", call);
@@ -328,14 +333,14 @@ namespace Sass {
     if (!call->block()) append_delimiter();
   }
 
-  void Inspect::operator()(Content* content)
+  void Inspect::operator()(Content_Ptr content)
   {
     append_indentation();
     append_token("@content", content);
     append_delimiter();
   }
 
-  void Inspect::operator()(Map* map)
+  void Inspect::operator()(Map_Ptr map)
   {
     if (output_style() == TO_SASS && map->empty()) {
       append_string("()");
@@ -355,7 +360,7 @@ namespace Sass {
     append_string(")");
   }
 
-  void Inspect::operator()(List* list)
+  void Inspect::operator()(List_Ptr list)
   {
     if (output_style() == TO_SASS && list->empty()) {
       append_string("()");
@@ -373,9 +378,8 @@ namespace Sass {
     if (output_style() == TO_SASS &&
         list->length() == 1 &&
         !list->from_selector() &&
-        !dynamic_cast<List*>((*list)[0]) &&
-        !dynamic_cast<List*>((*list)[0]) &&
-        !dynamic_cast<CommaSequence_Selector*>((*list)[0])) {
+        !SASS_MEMORY_CAST(List, list->at(0)) &&
+        !SASS_MEMORY_CAST(Selector_List, list->at(0))) {
       append_string("(");
     }
     else if (!in_declaration && (list->separator() == SASS_HASH ||
@@ -391,11 +395,11 @@ namespace Sass {
     for (size_t i = 0, L = list->size(); i < L; ++i) {
       if (list->separator() == SASS_HASH)
       { sep[0] = i % 2 ? ':' : ','; }
-      Expression* list_item = (*list)[i];
+      Expression_Obj list_item = list->at(i);
       if (output_style() != TO_SASS) {
         if (list_item->is_invisible()) {
           // this fixes an issue with "" in a list
-          if (!dynamic_cast<String_Constant*>(list_item)) {
+          if (!SASS_MEMORY_CAST(String_Constant, list_item)) {
             continue;
           }
         }
@@ -415,9 +419,8 @@ namespace Sass {
     if (output_style() == TO_SASS &&
         list->length() == 1 &&
         !list->from_selector() &&
-        !dynamic_cast<List*>((*list)[0]) &&
-        !dynamic_cast<List*>((*list)[0]) &&
-        !dynamic_cast<CommaSequence_Selector*>((*list)[0])) {
+        !SASS_MEMORY_CAST(List, list->at(0)) &&
+        !SASS_MEMORY_CAST(Selector_List, list->at(0))) {
       append_string(",)");
     }
     else if (!in_declaration && (list->separator() == SASS_HASH ||
@@ -429,7 +432,7 @@ namespace Sass {
 
   }
 
-  void Inspect::operator()(Binary_Expression* expr)
+  void Inspect::operator()(Binary_Expression_Ptr expr)
   {
     expr->left()->perform(this);
     if ( in_media_block ||
@@ -466,36 +469,36 @@ namespace Sass {
     expr->right()->perform(this);
   }
 
-  void Inspect::operator()(Unary_Expression* expr)
+  void Inspect::operator()(Unary_Expression_Ptr expr)
   {
     if (expr->type() == Unary_Expression::PLUS) append_string("+");
     else                                        append_string("-");
     expr->operand()->perform(this);
   }
 
-  void Inspect::operator()(Function_Call* call)
+  void Inspect::operator()(Function_Call_Ptr call)
   {
     append_token(call->name(), call);
     call->arguments()->perform(this);
   }
 
-  void Inspect::operator()(Function_Call_Schema* call)
+  void Inspect::operator()(Function_Call_Schema_Ptr call)
   {
     call->name()->perform(this);
     call->arguments()->perform(this);
   }
 
-  void Inspect::operator()(Variable* var)
+  void Inspect::operator()(Variable_Ptr var)
   {
     append_token(var->name(), var);
   }
 
-  void Inspect::operator()(Textual* txt)
+  void Inspect::operator()(Textual_Ptr txt)
   {
     append_token(txt->value(), txt);
   }
 
-  void Inspect::operator()(Number* n)
+  void Inspect::operator()(Number_Ptr n)
   {
 
     std::string res;
@@ -588,7 +591,7 @@ namespace Sass {
     else                return c;
   }
 
-  void Inspect::operator()(Color* c)
+  void Inspect::operator()(Color_Ptr c)
   {
     // output the final token
     std::stringstream ss;
@@ -607,7 +610,7 @@ namespace Sass {
 
     // get color from given name (if one was given at all)
     if (name != "" && name_to_color(name)) {
-      const Color* n = name_to_color(name);
+      Color_Ptr_Const n = name_to_color(name);
       r = Sass::round(cap_channel<0xff>(n->r()), opt.precision);
       g = Sass::round(cap_channel<0xff>(n->g()), opt.precision);
       b = Sass::round(cap_channel<0xff>(n->b()), opt.precision);
@@ -674,13 +677,13 @@ namespace Sass {
 
   }
 
-  void Inspect::operator()(Boolean* b)
+  void Inspect::operator()(Boolean_Ptr b)
   {
     // output the final token
     append_token(b->value() ? "true" : "false", b);
   }
 
-  void Inspect::operator()(String_Schema* ss)
+  void Inspect::operator()(String_Schema_Ptr ss)
   {
     // Evaluation should turn these into String_Constants,
     // so this method is only for inspection purposes.
@@ -691,12 +694,12 @@ namespace Sass {
     }
   }
 
-  void Inspect::operator()(String_Constant* s)
+  void Inspect::operator()(String_Constant_Ptr s)
   {
     append_token(s->value(), s);
   }
 
-  void Inspect::operator()(String_Quoted* s)
+  void Inspect::operator()(String_Quoted_Ptr s)
   {
     if (const char q = s->quote_mark()) {
       append_token(quote(s->value(), q), s);
@@ -705,17 +708,17 @@ namespace Sass {
     }
   }
 
-  void Inspect::operator()(Custom_Error* e)
+  void Inspect::operator()(Custom_Error_Ptr e)
   {
     append_token(e->message(), e);
   }
 
-  void Inspect::operator()(Custom_Warning* w)
+  void Inspect::operator()(Custom_Warning_Ptr w)
   {
     append_token(w->message(), w);
   }
 
-  void Inspect::operator()(Supports_Operator* so)
+  void Inspect::operator()(Supports_Operator_Ptr so)
   {
 
     if (so->needs_parens(so->left())) append_string("(");
@@ -737,16 +740,16 @@ namespace Sass {
     if (so->needs_parens(so->right())) append_string(")");
   }
 
-  void Inspect::operator()(Supports_Negation* sn)
+  void Inspect::operator()(Supports_Negation_Ptr sn)
   {
     append_token("not", sn);
     append_mandatory_space();
-    if (sn->needs_parens(sn->condition())) append_string("(");
+    if (sn->needs_parens(&sn->condition())) append_string("(");
     sn->condition()->perform(this);
-    if (sn->needs_parens(sn->condition())) append_string(")");
+    if (sn->needs_parens(&sn->condition())) append_string(")");
   }
 
-  void Inspect::operator()(Supports_Declaration* sd)
+  void Inspect::operator()(Supports_Declaration_Ptr sd)
   {
     append_string("(");
     sd->feature()->perform(this);
@@ -755,15 +758,15 @@ namespace Sass {
     append_string(")");
   }
 
-  void Inspect::operator()(Supports_Interpolation* sd)
+  void Inspect::operator()(Supports_Interpolation_Ptr sd)
   {
     sd->value()->perform(this);
   }
 
-  void Inspect::operator()(Media_Query* mq)
+  void Inspect::operator()(Media_Query_Ptr mq)
   {
     size_t i = 0;
-    if (mq->media_type()) {
+    if (&mq->media_type()) {
       if      (mq->is_negated())    append_string("not ");
       else if (mq->is_restricted()) append_string("only ");
       mq->media_type()->perform(this);
@@ -777,7 +780,7 @@ namespace Sass {
     }
   }
 
-  void Inspect::operator()(Media_Query_Expression* mqe)
+  void Inspect::operator()(Media_Query_Expression_Ptr mqe)
   {
     if (mqe->is_interpolated()) {
       mqe->feature()->perform(this);
@@ -793,7 +796,7 @@ namespace Sass {
     }
   }
 
-  void Inspect::operator()(At_Root_Query* ae)
+  void Inspect::operator()(At_Root_Query_Ptr ae)
   {
     append_string("(");
     ae->feature()->perform(this);
@@ -804,14 +807,14 @@ namespace Sass {
     append_string(")");
   }
 
-  void Inspect::operator()(Null* n)
+  void Inspect::operator()(Null_Ptr n)
   {
     // output the final token
     append_token("null", n);
   }
 
   // parameters and arguments
-  void Inspect::operator()(Parameter* p)
+  void Inspect::operator()(Parameter_Ptr p)
   {
     append_token(p->name(), p);
     if (p->default_value()) {
@@ -823,7 +826,7 @@ namespace Sass {
     }
   }
 
-  void Inspect::operator()(Parameters* p)
+  void Inspect::operator()(Parameters_Ptr p)
   {
     append_string("(");
     if (!p->empty()) {
@@ -836,26 +839,29 @@ namespace Sass {
     append_string(")");
   }
 
-  void Inspect::operator()(Argument* a)
+  void Inspect::operator()(Argument_Ptr a)
   {
     if (!a->name().empty()) {
       append_token(a->name(), a);
       append_colon_separator();
     }
+    if (!a->value()) return;
     // Special case: argument nulls can be ignored
     if (a->value()->concrete_type() == Expression::NULL_VAL) {
       return;
     }
     if (a->value()->concrete_type() == Expression::STRING) {
-      String_Constant* s = static_cast<String_Constant*>(a->value());
-      s->perform(this);
-    } else a->value()->perform(this);
+      String_Constant_Ptr s = SASS_MEMORY_CAST(String_Constant, a->value());
+      if (s) s->perform(this);
+    } else {
+      a->value()->perform(this);
+    }
     if (a->is_rest_argument()) {
       append_string("...");
     }
   }
 
-  void Inspect::operator()(Arguments* a)
+  void Inspect::operator()(Arguments_Ptr a)
   {
     append_string("(");
     if (!a->empty()) {
@@ -869,17 +875,17 @@ namespace Sass {
     append_string(")");
   }
 
-  void Inspect::operator()(Selector_Schema* s)
+  void Inspect::operator()(Selector_Schema_Ptr s)
   {
     s->contents()->perform(this);
   }
 
-  void Inspect::operator()(Parent_Selector* p)
+  void Inspect::operator()(Parent_Selector_Ptr p)
   {
     if (p->is_real_parent_ref()) append_string("&");
   }
 
-  void Inspect::operator()(Placeholder_Selector* s)
+  void Inspect::operator()(Placeholder_Selector_Ptr s)
   {
     append_token(s->name(), s);
     if (s->has_line_break()) append_optional_linefeed();
@@ -887,33 +893,33 @@ namespace Sass {
 
   }
 
-  void Inspect::operator()(Element_Selector* s)
+  void Inspect::operator()(Element_Selector_Ptr s)
   {
     append_token(s->ns_name(), s);
   }
 
-  void Inspect::operator()(Class_Selector* s)
+  void Inspect::operator()(Class_Selector_Ptr s)
   {
     append_token(s->ns_name(), s);
     if (s->has_line_break()) append_optional_linefeed();
     if (s->has_line_break()) append_indentation();
   }
 
-  void Inspect::operator()(Id_Selector* s)
+  void Inspect::operator()(Id_Selector_Ptr s)
   {
     append_token(s->ns_name(), s);
     if (s->has_line_break()) append_optional_linefeed();
     if (s->has_line_break()) append_indentation();
   }
 
-  void Inspect::operator()(Attribute_Selector* s)
+  void Inspect::operator()(Attribute_Selector_Ptr s)
   {
     append_string("[");
     add_open_mapping(s);
     append_token(s->ns_name(), s);
     if (!s->matcher().empty()) {
       append_string(s->matcher());
-      if (s->value()) {
+      if (&s->value() && *s->value()) {
         s->value()->perform(this);
       }
     }
@@ -921,7 +927,7 @@ namespace Sass {
     append_string("]");
   }
 
-  void Inspect::operator()(Pseudo_Selector* s)
+  void Inspect::operator()(Pseudo_Selector_Ptr s)
   {
     append_token(s->ns_name(), s);
     if (s->expression()) {
@@ -931,7 +937,7 @@ namespace Sass {
     }
   }
 
-  void Inspect::operator()(Wrapped_Selector* s)
+  void Inspect::operator()(Wrapped_Selector_Ptr s)
   {
     bool was = in_wrapped;
     in_wrapped = true;
@@ -945,7 +951,7 @@ namespace Sass {
     in_wrapped = was;
   }
 
-  void Inspect::operator()(SimpleSequence_Selector* s)
+  void Inspect::operator()(Compound_Selector_Ptr s)
   {
     for (size_t i = 0, L = s->length(); i < L; ++i) {
       (*s)[i]->perform(this);
@@ -957,13 +963,13 @@ namespace Sass {
     }
   }
 
-  void Inspect::operator()(Sequence_Selector* c)
+  void Inspect::operator()(Complex_Selector_Ptr c)
   {
-    SimpleSequence_Selector*      head = c->head();
-    Sequence_Selector*            tail = c->tail();
-    Sequence_Selector::Combinator comb = c->combinator();
+    Compound_Selector_Obj      head = c->head();
+    Complex_Selector_Obj            tail = c->tail();
+    Complex_Selector::Combinator comb = c->combinator();
 
-    if (comb == Sequence_Selector::ANCESTOR_OF && (!head || head->empty())) {
+    if (comb == Complex_Selector::ANCESTOR_OF && (!head || head->empty())) {
       if (tail) tail->perform(this);
       return;
     }
@@ -978,30 +984,30 @@ namespace Sass {
     if (head && head->length() != 0) head->perform(this);
     bool is_empty = !head || head->length() == 0 || head->is_empty_reference();
     bool is_tail = head && !head->is_empty_reference() && tail;
-    if (output_style() == COMPRESSED && comb != Sequence_Selector::ANCESTOR_OF) scheduled_space = 0;
+    if (output_style() == COMPRESSED && comb != Complex_Selector::ANCESTOR_OF) scheduled_space = 0;
 
     switch (comb) {
-      case Sequence_Selector::ANCESTOR_OF:
+      case Complex_Selector::ANCESTOR_OF:
         if (is_tail) append_mandatory_space();
       break;
-      case Sequence_Selector::PARENT_OF:
+      case Complex_Selector::PARENT_OF:
         append_optional_space();
         append_string(">");
         append_optional_space();
       break;
-      case Sequence_Selector::ADJACENT_TO:
+      case Complex_Selector::ADJACENT_TO:
         append_optional_space();
         append_string("+");
         append_optional_space();
       break;
-      case Sequence_Selector::REFERENCE:
+      case Complex_Selector::REFERENCE:
         append_mandatory_space();
         append_string("/");
         c->reference()->perform(this);
         append_string("/");
         append_mandatory_space();
       break;
-      case Sequence_Selector::PRECEDES:
+      case Complex_Selector::PRECEDES:
         if (is_empty) append_optional_space();
         else append_mandatory_space();
         append_string("~");
@@ -1009,7 +1015,7 @@ namespace Sass {
         else append_optional_space();
       break;
     }
-    if (tail && comb != Sequence_Selector::ANCESTOR_OF) {
+    if (tail && comb != Complex_Selector::ANCESTOR_OF) {
       if (c->has_line_break()) append_optional_linefeed();
     }
     if (tail) tail->perform(this);
@@ -1020,7 +1026,7 @@ namespace Sass {
     }
   }
 
-  void Inspect::operator()(CommaSequence_Selector* g)
+  void Inspect::operator()(Selector_List_Ptr g)
   {
 
     if (g->empty()) {
@@ -1034,8 +1040,8 @@ namespace Sass {
     bool was_comma_array = in_comma_array;
     // probably ruby sass eqivalent of element_needs_parens
     if (output_style() == TO_SASS && g->length() == 1 &&
-      (!dynamic_cast<List*>((*g)[0]) &&
-       !dynamic_cast<CommaSequence_Selector*>((*g)[0]))) {
+      (!SASS_MEMORY_CAST(List, (*g)[0]) &&
+       !SASS_MEMORY_CAST(Selector_List, (*g)[0]))) {
       append_string("(");
     }
     else if (!in_declaration && in_comma_array) {
@@ -1047,7 +1053,7 @@ namespace Sass {
     for (size_t i = 0, L = g->length(); i < L; ++i) {
       if (!in_wrapped && i == 0) append_indentation();
       if ((*g)[i] == 0) continue;
-      schedule_mapping((*g)[i]->last());
+      schedule_mapping(&g->at(i)->last());
       // add_open_mapping((*g)[i]->last());
       (*g)[i]->perform(this);
       // add_close_mapping((*g)[i]->last());
@@ -1060,8 +1066,8 @@ namespace Sass {
     in_comma_array = was_comma_array;
     // probably ruby sass eqivalent of element_needs_parens
     if (output_style() == TO_SASS && g->length() == 1 &&
-      (!dynamic_cast<List*>((*g)[0]) &&
-       !dynamic_cast<CommaSequence_Selector*>((*g)[0]))) {
+      (!SASS_MEMORY_CAST(List, (*g)[0]) &&
+       !SASS_MEMORY_CAST(Selector_List, (*g)[0]))) {
       append_string(",)");
     }
     else if (!in_declaration && in_comma_array) {
@@ -1070,7 +1076,7 @@ namespace Sass {
 
   }
 
-  void Inspect::fallback_impl(AST_Node* n)
+  void Inspect::fallback_impl(AST_Node_Ptr n)
   {
   }
 
