@@ -438,30 +438,15 @@ namespace Sass {
       return normalized[0] == '.' ? normalized.insert(0, prefix) : normalized;
     }
 
-    // compress a color sixtuplet if possible
-    // input: "#CC9900" -> output: "#C90"
-    std::string normalize_sixtuplet(const std::string& col) {
-      if(
-        col.substr(1, 1) == col.substr(2, 1) &&
-        col.substr(3, 1) == col.substr(4, 1) &&
-        col.substr(5, 1) == col.substr(6, 1)
-      ) {
-        return std::string("#" + col.substr(1, 1)
-                          + col.substr(3, 1)
-                          + col.substr(5, 1));
-      } else {
-        return std::string(col);
-      }
-    }
-
-    bool isPrintable(Ruleset* r, Sass_Output_Style style) {
+    bool isPrintable(Ruleset_Ptr r, Sass_Output_Style style) {
       if (r == NULL) {
         return false;
       }
 
-      Block* b = r->block();
+      Block_Obj b = r->block();
 
-      bool hasSelectors = static_cast<CommaSequence_Selector*>(r->selector())->length() > 0;
+      Selector_List_Ptr sl = SASS_MEMORY_CAST(Selector_List, r->selector());
+      bool hasSelectors = sl ? sl->length() > 0 : false;
 
       if (!hasSelectors) {
         return false;
@@ -470,17 +455,17 @@ namespace Sass {
       bool hasDeclarations = false;
       bool hasPrintableChildBlocks = false;
       for (size_t i = 0, L = b->length(); i < L; ++i) {
-        Statement* stm = (*b)[i];
-        if (dynamic_cast<Directive*>(stm)) {
+        Statement_Obj stm = b->at(i);
+        if (dynamic_cast<Directive_Ptr>(&stm)) {
           return true;
-        } else if (Declaration* d = dynamic_cast<Declaration*>(stm)) {
+        } else if (Declaration_Ptr d = dynamic_cast<Declaration_Ptr>(&stm)) {
           return isPrintable(d, style);
-        } else if (dynamic_cast<Has_Block*>(stm)) {
-          Block* pChildBlock = ((Has_Block*)stm)->block();
-          if (isPrintable(pChildBlock, style)) {
+        } else if (dynamic_cast<Has_Block_Ptr>(&stm)) {
+          Block_Obj pChildBlock = ((Has_Block_Ptr)&stm)->block();
+          if (isPrintable(&pChildBlock, style)) {
             hasPrintableChildBlocks = true;
           }
-        } else if (Comment* c = dynamic_cast<Comment*>(stm)) {
+        } else if (Comment_Ptr c = dynamic_cast<Comment_Ptr>(&stm)) {
           // keep for uncompressed
           if (style != COMPRESSED) {
             hasDeclarations = true;
@@ -501,43 +486,43 @@ namespace Sass {
       return false;
     }
 
-    bool isPrintable(String_Constant* s, Sass_Output_Style style)
+    bool isPrintable(String_Constant_Ptr s, Sass_Output_Style style)
     {
       return ! s->value().empty();
     }
 
-    bool isPrintable(String_Quoted* s, Sass_Output_Style style)
+    bool isPrintable(String_Quoted_Ptr s, Sass_Output_Style style)
     {
       return true;
     }
 
-    bool isPrintable(Declaration* d, Sass_Output_Style style)
+    bool isPrintable(Declaration_Ptr d, Sass_Output_Style style)
     {
-      Expression* val = d->value();
-      if (String_Quoted* sq = dynamic_cast<String_Quoted*>(val)) return isPrintable(sq, style);
-      if (String_Constant* sc = dynamic_cast<String_Constant*>(val)) return isPrintable(sc, style);
+      Expression_Obj val = d->value();
+      if (String_Quoted_Obj sq = SASS_MEMORY_CAST(String_Quoted, val)) return isPrintable(&sq, style);
+      if (String_Constant_Obj sc = SASS_MEMORY_CAST(String_Constant, val)) return isPrintable(&sc, style);
       return true;
     }
 
-    bool isPrintable(Supports_Block* f, Sass_Output_Style style) {
+    bool isPrintable(Supports_Block_Ptr f, Sass_Output_Style style) {
       if (f == NULL) {
         return false;
       }
 
-      Block* b = f->block();
+      Block_Obj b = f->block();
 
-//      bool hasSelectors = f->selector() && static_cast<CommaSequence_Selector*>(f->selector())->length() > 0;
+//      bool hasSelectors = f->selector() && static_cast<Selector_List_Ptr>(f->selector())->length() > 0;
 
       bool hasDeclarations = false;
       bool hasPrintableChildBlocks = false;
       for (size_t i = 0, L = b->length(); i < L; ++i) {
-        Statement* stm = (*b)[i];
-        if (typeid(*stm) == typeid(Declaration) || typeid(*stm) == typeid(Directive)) {
+        Statement_Obj stm = b->at(i);
+        if (dynamic_cast<Declaration_Ptr>(&stm) || dynamic_cast<Directive_Ptr>(&stm)) {
           hasDeclarations = true;
         }
-        else if (dynamic_cast<Has_Block*>(stm)) {
-          Block* pChildBlock = ((Has_Block*)stm)->block();
-          if (isPrintable(pChildBlock, style)) {
+        else if (dynamic_cast<Has_Block_Ptr>(&stm)) {
+          Block_Obj pChildBlock = ((Has_Block_Ptr)&stm)->block();
+          if (isPrintable(&pChildBlock, style)) {
             hasPrintableChildBlocks = true;
           }
         }
@@ -550,47 +535,47 @@ namespace Sass {
       return false;
     }
 
-    bool isPrintable(Media_Block* m, Sass_Output_Style style)
+    bool isPrintable(Media_Block_Ptr m, Sass_Output_Style style)
     {
       if (m == 0) return false;
-      Block* b = m->block();
+      Block_Obj b = m->block();
       if (b == 0) return false;
       for (size_t i = 0, L = b->length(); i < L; ++i) {
-        Statement* stm = (*b)[i];
-        if (typeid(*stm) == typeid(Directive)) return true;
-        else if (typeid(*stm) == typeid(Declaration)) return true;
-        else if (typeid(*stm) == typeid(Comment)) {
-          Comment* c = (Comment*) stm;
+        Statement_Obj stm = b->at(i);
+        if (dynamic_cast<Directive_Ptr>(&stm)) return true;
+        else if (dynamic_cast<Declaration_Ptr>(&stm)) return true;
+        else if (dynamic_cast<Comment_Ptr>(&stm)) {
+          Comment_Ptr c = (Comment_Ptr) &stm;
           if (isPrintable(c, style)) {
             return true;
           }
         }
-        else if (typeid(*stm) == typeid(Ruleset)) {
-          Ruleset* r = (Ruleset*) stm;
+        else if (dynamic_cast<Ruleset_Ptr>(&stm)) {
+          Ruleset_Ptr r = (Ruleset_Ptr) &stm;
           if (isPrintable(r, style)) {
             return true;
           }
         }
-        else if (typeid(*stm) == typeid(Supports_Block)) {
-          Supports_Block* f = (Supports_Block*) stm;
+        else if (dynamic_cast<Supports_Block_Ptr>(&stm)) {
+          Supports_Block_Ptr f = (Supports_Block_Ptr) &stm;
           if (isPrintable(f, style)) {
             return true;
           }
         }
-        else if (typeid(*stm) == typeid(Media_Block)) {
-          Media_Block* m = (Media_Block*) stm;
+        else if (dynamic_cast<Media_Block_Ptr>(&stm)) {
+          Media_Block_Ptr m = (Media_Block_Ptr) &stm;
           if (isPrintable(m, style)) {
             return true;
           }
         }
-        else if (dynamic_cast<Has_Block*>(stm) && isPrintable(((Has_Block*)stm)->block(), style)) {
+        else if (dynamic_cast<Has_Block_Ptr>(&stm) && isPrintable(((Has_Block_Ptr)&stm)->block(), style)) {
           return true;
         }
       }
       return false;
     }
 
-    bool isPrintable(Comment* c, Sass_Output_Style style)
+    bool isPrintable(Comment_Ptr c, Sass_Output_Style style)
     {
       // keep for uncompressed
       if (style != COMPRESSED) {
@@ -604,41 +589,41 @@ namespace Sass {
       return false;
     };
 
-    bool isPrintable(Block* b, Sass_Output_Style style) {
-      if (b == NULL) {
+    bool isPrintable(Block_Obj b, Sass_Output_Style style) {
+      if (!b) {
         return false;
       }
 
       for (size_t i = 0, L = b->length(); i < L; ++i) {
-        Statement* stm = (*b)[i];
-        if (typeid(*stm) == typeid(Declaration) || typeid(*stm) == typeid(Directive)) {
+        Statement_Obj stm = b->at(i);
+        if (dynamic_cast<Declaration_Ptr>(&stm) || dynamic_cast<Directive_Ptr>(&stm)) {
           return true;
         }
-        else if (typeid(*stm) == typeid(Comment)) {
-          Comment* c = (Comment*) stm;
+        else if (dynamic_cast<Comment_Ptr>(&stm)) {
+          Comment_Ptr c = (Comment_Ptr) &stm;
           if (isPrintable(c, style)) {
             return true;
           }
         }
-        else if (typeid(*stm) == typeid(Ruleset)) {
-          Ruleset* r = (Ruleset*) stm;
+        else if (dynamic_cast<Ruleset_Ptr>(&stm)) {
+          Ruleset_Ptr r = (Ruleset_Ptr) &stm;
           if (isPrintable(r, style)) {
             return true;
           }
         }
-        else if (typeid(*stm) == typeid(Supports_Block)) {
-          Supports_Block* f = (Supports_Block*) stm;
+        else if (dynamic_cast<Supports_Block_Ptr>(&stm)) {
+          Supports_Block_Ptr f = (Supports_Block_Ptr) &stm;
           if (isPrintable(f, style)) {
             return true;
           }
         }
-        else if (typeid(*stm) == typeid(Media_Block)) {
-          Media_Block* m = (Media_Block*) stm;
+        else if (dynamic_cast<Media_Block_Ptr>(&stm)) {
+          Media_Block_Ptr m = (Media_Block_Ptr) &stm;
           if (isPrintable(m, style)) {
             return true;
           }
         }
-        else if (dynamic_cast<Has_Block*>(stm) && isPrintable(((Has_Block*)stm)->block(), style)) {
+        else if (dynamic_cast<Has_Block_Ptr>(&stm) && isPrintable(((Has_Block_Ptr)&stm)->block(), style)) {
           return true;
         }
       }
@@ -646,27 +631,9 @@ namespace Sass {
       return false;
     }
 
-    std::string vecJoin(const std::vector<std::string>& vec, const std::string& sep)
-    {
-      switch (vec.size())
-      {
-        case 0:
-            return std::string("");
-        case 1:
-            return vec[0];
-        default:
-            std::ostringstream os;
-            os << vec[0];
-            for (size_t i = 1; i < vec.size(); i++) {
-              os << sep << vec[i];
-            }
-            return os.str();
-      }
+    bool isAscii(const char chr) {
+      return unsigned(chr) < 128;
     }
 
-     bool isAscii(const char chr) {
-       return unsigned(chr) < 128;
-     }
-
   }
 }
